#!/usr/bin/python3

# **************************************************************************** #
#                                                                              #
#                                                         :::      ::::::::    #
#    cli.py                                             :+:      :+:    :+:    #
#                                                     +:+ +:+         +:+      #
#    By: atrouill <atrouill@student.42.fr>          +#+  +:+       +#+         #
#                                                 +#+#+#+#+#+   +#+            #
#    Created: 2021/01/24 16:01:19 by atrouill          #+#    #+#              #
#    Updated: 2021/01/24 16:01:20 by atrouill         ###   ########.fr        #
#                                                                              #
# **************************************************************************** #

from colorama import Fore, Style

def	input_bool(prompt: str) -> bool:
	while True:
		res = input(prompt)
		if not res or res.lower() == "n" or res.lower() == "no":
			return False
		elif res.lower() == "y" or res.lower() == "yes":
			return True
		else:
			continue

def obtain_parameters() -> dict:
	"""
	Obtain parameters from the user
	"""
	params = dict()
	print(Fore.CYAN + "MAKEFILE CONFIGURATION :" + Style.RESET_ALL)
	print(Style.BRIGHT + "A few questions to configure your Makefile as well as possible!" + Style.RESET_ALL)
	params["target"] = input("\tProgramm name : ")
	params["bin_folder"] = input("\tBinary folder name : ")
	params["src_folder"] = input("\tSource folder name : ")
	params["inc_folder"] = input("\tInclude folder name : ")
	print("\tLibraries : ")
	params["library_libft"] = input_bool("\t\tlibftprintf [y/N] : ")
	if params["library_libft"]:
		params["folder_libft"] = input("\t\t\tLibftprintf folder : ")
	params["library_mlx"] = input_bool("\t\tminilibx [y/N] : ")
	if params["library_mlx"]:
		params["compile_mlx"] = input_bool("\t\t\tDoes minilibx need to compile with your project? [y/N] : ")
		if params["compile_mlx"]:
			params["folder_mlx"] = input("\t\t\tMinilibx folder : ")
	params["library"] = input("\t\tOther argument for library (empty if none) : ")

	return params



#!/usr/bin/python3

# **************************************************************************** #
#                                                                              #
#                                                         :::      ::::::::    #
#    genmake.py                                         :+:      :+:    :+:    #
#                                                     +:+ +:+         +:+      #
#    By: atrouill <atrouill@student.42.fr>          +#+  +:+       +#+         #
#                                                 +#+#+#+#+#+   +#+            #
#    Created: 2021/01/22 00:47:02 by atrouill          #+#    #+#              #
#    Updated: 2021/01/22 00:47:06 by atrouill         ###   ########.fr        #
#                                                                              #
# **************************************************************************** #

from pathlib import Path
from colorama import Fore, Style

from .source_file_maker import populate_sources
from .args import parse_args
from .cli import obtain_parameters
from .makefile import generate_makefile


def create_makefile(params: dict):
	"""
	Create Makefile and fill it with user params
	"""
	content = generate_makefile(params)
	with open("Makefile", "w") as file:
		file.write(content)
		file.close()
	print(Fore.GREEN + "✅ Makefile created!" + Style.RESET_ALL)



def main():
	args = parse_args()
	p = Path('Makefile')
	if (args.remake == True) or not p.exists():
		params = obtain_parameters()
		create_makefile(params)
	populate_sources()




#!/bin/usr/python3

# **************************************************************************** #
#                                                                              #
#                                                         :::      ::::::::    #
#    args.py                                            :+:      :+:    :+:    #
#                                                     +:+ +:+         +:+      #
#    By: atrouill <atrouill@student.42.fr>          +#+  +:+       +#+         #
#                                                 +#+#+#+#+#+   +#+            #
#    Created: 2021/01/24 13:49:19 by atrouill          #+#    #+#              #
#    Updated: 2021/01/24 13:49:20 by atrouill         ###   ########.fr        #
#                                                                              #
# **************************************************************************** #

import genmake.config as config
import argparse

def	parse_args():
	parser = argparse.ArgumentParser(description=config.DESC)
	parser = argparse.ArgumentParser(prog=config.PROG)
	parser.add_argument('--remake',
		dest="remake",
		default=False,
		action="store_true",
		help='Delete and reconstruct Makefile'
	)
	parser.add_argument('--version',
		action='version',
		version='%(prog)s ' + config.VERSION
	)
	args = parser.parse_args()
	
	return args




#!/bin/usr/python3

# **************************************************************************** #
#                                                                              #
#                                                         :::      ::::::::    #
#    build_rules.py                                     :+:      :+:    :+:    #
#                                                     +:+ +:+         +:+      #
#    By: atrouill <atrouill@student.42.fr>          +#+  +:+       +#+         #
#                                                 +#+#+#+#+#+   +#+            #
#    Created: 2021/01/23 19:15:47 by atrouill          #+#    #+#              #
#    Updated: 2021/01/23 19:15:48 by atrouill         ###   ########.fr        #
#                                                                              #
# **************************************************************************** #

def	clean(params: dict) -> str:
	"""
	Build clean rules for Makefile
	"""
	clean = "\t@$(RM) -rf $(BUILDDIR)\n"

	if params["library_libft"]:
		clean += "\t@make $@ -s -C " + params["folder_libft"] + "\n"
	if params["library_mlx"] and params["compile_mlx"]:
		clean += "\t@make $@ -s -C " + params["folder_mlx"] + "\n"

	return clean

def	fclean(params: dict) -> str:
	"""
	Build fclean rules for Makefile
	"""
	fclean = str()

	if params["bin_folder"] == ".":
		fclean += "\t@$(RM) -rf $(TARGET)\n"
	else:
		fclean += "\t@$(RM) -rf $(TARGETDIR)\n"
	if params["library_libft"]:
		fclean += "\t@make $@ -s -C " + params["folder_libft"] + "\n"

	return fclean

def lib(params: dict) -> str:
	"""
	Build rules for make lib
	"""
	rules = str()

	if params["library_libft"]:
		rules += "libft:\n"
		rules += "\t@make -s -C " + params["folder_libft"] + "\n"
	if params["library_mlx"] and params["compile_mlx"]:
		rules += "\nminilibx:\n"
		rules += "\t@make -s -C " + params["folder_mlx"] + "\n"

	return rules

def all(params: dict) -> str:
	"""
	Build rules for all with all lib
	"""
	all_rules = "directories"

	if params["library_libft"]:
		all_rules += " libft"
	if params["library_mlx"] and params["compile_mlx"]:
		all_rules += " minilibx"
	all_rules += " $(TARGET)"

	return all_rules

def bonus(params: dict) -> str:
	"""
	Build rules for bonus with all lib
	"""
	all_rules = "directories"

	if params["library_libft"]:
		all_rules += " libft"
	if params["library_mlx"] and params["compile_mlx"]:
		all_rules += " minilibx"
	all_rules += " $(TARGET_BONUS)"

	return all_rules

def lib_inc(params: dict) -> str:
	"""
	Build lib inc for linker settings
	"""
	lib_inc = str()

	if params["library_libft"]:
		lib_inc += " -L" + params["folder_libft"] + " -lftprintf"
	if params["library_mlx"] and params["compile_mlx"]:
		lib_inc += " -L" + params["folder_mlx"] + " -lmlx"
	if params["library"]:
		lib_inc += " " + params["library"]

	lib_inc += "\n"

	return lib_inc

def phony(params: dict) -> str:
	"""
	Build phony rules according to 42 rules
	"""
	phony = "all re clean fclean norm bonus"

	if params["library_libft"]:
		phony += " libft"
	if params["library_mlx"] and params["compile_mlx"]:
		phony += " minilibx"

	return phony



#!/usr/bin/python3

# **************************************************************************** #
#                                                                              #
#                                                         :::      ::::::::    #
#    source_file_maker.py                               :+:      :+:    :+:    #
#                                                     +:+ +:+         +:+      #
#    By: atrouill <atrouill@student.42.fr>          +#+  +:+       +#+         #
#                                                 +#+#+#+#+#+   +#+            #
#    Created: 2021/01/21 23:34:32 by atrouill          #+#    #+#              #
#    Updated: 2021/01/22 00:47:16 by atrouill         ###   ########.fr        #
#                                                                              #
# **************************************************************************** #

import os
import pathlib
import fileinput

from colorama.ansi import Fore, Style

sources_file_path = "./sources.mk"

def obtain_list_file(path, extension):
	p = path.glob('*')
	files = [x for x in p if x.is_file() and x.suffix == extension]
	return files

def obtain_list_dir(src_folder: str):
	p = pathlib.Path(r'.').glob(src_folder + '/**')
	dir = [x for x in p if x.is_dir()]
	return dir

def write_comment(file, text: str):
	file.write("## " + text.upper() + " ##\n\n")

def write_file_list(source_file, files):
	source_file.write("SOURCES\t\t+=\n")
	for file in files:
		source_file.write("\t\t" + str(file) + "\n")
	source_file.write("\n\n")

def write_file_list_bonus(source_file, files):
	source_file.write("SOURCES_BONUS\t+= $(SOURCES)\n")
	for file in files:
		source_file.write("\t\t" + str(file) + "\n")
	source_file.write("\n\n")

def obtain_bonus(src_folder):
	p = pathlib.Path(src_folder).glob('**/*_bonus.c')
	return (list(p))

def build_source_file(source_file):
	srcs_folder = obtain_src_folder()
	dirs = obtain_list_dir(srcs_folder)
	for dir in dirs:
		files = obtain_list_file(dir, ".c")
		if files:
			write_comment(source_file, str(dir))
			write_file_list(source_file, files)
	bonus = obtain_bonus(srcs_folder)
	if bonus:
		write_comment(source_file, "BONUS")
		write_file_list_bonus(source_file, bonus)


def obtain_max_lenght(source_file):
	max_length = 0
	source_file.seek(0, os.SEEK_SET)
	for line in source_file:
		if(len(line) > max_length):
	  	  max_length = len(line)
	return max_length

def beautify_file(sources_file_path, len: int):
	"""
	Check file if everything is aligned
	"""
	for line in fileinput.input(sources_file_path, inplace=True):
		if ".c" in line:
			print('{0:{len}}{1}'.format(line.rstrip(), "\t\\\n", len=len), end='')
		elif "+=" in line and "_BONUS" not in line:
			print('{0:{len}}{1}'.format(line.rstrip(), "\t\t\\\n", len=len), end='')
		elif "SOURCES_BONUS" in line:
			print('{0:{len}}{1}'.format(line.rstrip(), "\t\t\t\\\n", len=len), end='')
		else:
			print(line, end='')

def create_source_file():
	"""
	Create sources.mk
	"""
	p = pathlib.Path(sources_file_path)
	if p.exists():
		os.remove(sources_file_path)
	source_file = open(sources_file_path, 'w+')
	write_comment(source_file, p.name)
	return source_file

def obtain_src_folder():
	"""
	Parse Makefile and return the SRCDIR configured by user
	"""
	with open("Makefile", 'r') as file:
		for line in file:
			if "SRCDIR" in line:
				src_folder = line.split()
				file.close()
				return src_folder[-1]

def populate_sources():
	"""
	List all .c files and create sources.mk
	"""
	source_file = create_source_file()

	build_source_file(source_file)
	max_lenght = obtain_max_lenght(source_file)
	beautify_file(sources_file_path, max_lenght + 5)
	source_file.close()
	print(Fore.GREEN + "✅ Makefile updated!" + Style.RESET_ALL)
